@@ -40,6 +40,8 @@ class LeanREPLConfig:
         build_repl: bool = True,
         lake_path: str | PathLike = "lake",
         memory_hard_limit_mb: int | None = None,
+        enable_incremental_optimization: bool = True,
+        enable_parallel_elaboration: bool = True,
         verbose: bool = False,
     ):
         """
@@ -82,6 +84,13 @@ class LeanREPLConfig:
                 The maximum memory usage in MB for the Lean server. Setting this value too low may lead to more command processing failures.
                 Only available on Linux platforms.
                 Default is `None`, which means no limit.
+            enable_incremental_optimization:
+                Whether to enable incremental optimization for all commands in the Lean REPL. This can significantly speed up processing
+                and decrease memory usage of commands by automatically reusing partial computations from previous commands.
+                Only available for Lean >= v4.8.0-rc1. Default is `True`.
+            enable_parallel_elaboration:
+                Whether to enable parallel elaboration in the Lean REPL. This can significantly speed up processing
+                of commands, especially in large files. Only available for Lean >= v4.19.0. Default is `True`.
             verbose:
                 Whether to print additional information during the setup process.
 
@@ -128,6 +137,8 @@ class LeanREPLConfig:
         self.local_repl_path = Path(local_repl_path) if local_repl_path else None
         self.build_repl = build_repl
         self.memory_hard_limit_mb = memory_hard_limit_mb
+        self.enable_incremental_optimization = enable_incremental_optimization
+        self.enable_parallel_elaboration = enable_parallel_elaboration
         self.lake_path = Path(lake_path)
         self.verbose = verbose
         self._timeout_lock = 300
@@ -383,16 +394,27 @@ class LeanREPLConfig:
         check_lake(self.lake_path)
 
         try:
-            subprocess.run(
+            # Capture build output so failures can be diagnosed
+            res = subprocess.run(
                 [str(self.lake_path), "build"],
                 cwd=self._cache_repl_dir,
-                check=True,
-                stdout=None if self.verbose else subprocess.DEVNULL,
-                stderr=None if self.verbose else subprocess.DEVNULL,
+                stdout=None if self.verbose else subprocess.PIPE,
+                stderr=None if self.verbose else subprocess.PIPE,
+                text=True,
+            )
+        except FileNotFoundError as e:
+            raise RuntimeError(
+                f"Lean 4 build system executable not found at `{self.lake_path}`. "
+                "You can try to run `install-lean` or follow: https://leanprover-community.github.io/get_started.html"
+            ) from e
+
+        if res.returncode != 0:
+            out = res.stdout or ""
+            err = res.stderr or ""
+            raise RuntimeError(
+                f"Failed to build the REPL at {self._cache_repl_dir}"
+                f"\n{'-' * 50}\nstdout:\n{out}\n{'-' * 50}\nstderr:\n{err}\n{'-' * 50}"
             )
-        except subprocess.CalledProcessError as e:
-            logger.error("Failed to build the REPL at %s: %s", self._cache_repl_dir, e)
-            raise
 
     def _get_available_lean_versions(self) -> list[tuple[str, str | None]]:
         """
